/*****************************************************************************/
/* Copyright (C) 2003-2005, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/* This software may only be used under the terms of the JRS MPEG-7 Library  */
/* license, available at                                                     */
/* http://iis.joanneum.at/MPEG-7/JRS-MPEG-7-Lib-License.pdf                  */
/*****************************************************************************/


// -*- c++ -*-

#ifndef MP7JRS_ENUMERATOR_INCLUDED
#define MP7JRS_ENUMERATOR_INCLUDED

//#include "Mp7JrsValueVectorOf.h"
#include "Mp7JrsPtr.h"

class Mp7JrsNode;

// Allocation and deallocation have to happen in the library so we can
// pass objects over DLL boundaries.
MP7JRS_EXPORT Mp7JrsPtr<Mp7JrsNode> * allocateVector(unsigned int size);

void MP7JRS_EXPORT deallocateVector(Mp7JrsPtr<Mp7JrsNode> *vect);


/** Enumerator template */
template<class C>
class Mp7JrsEnumerator {

public:

	Mp7JrsEnumerator()
		: content_length(0)
		, current_outpos(0)
		, max_outpos(0)
		, content(0)
	{
		content = allocateVector(20);
		content_length = 20;
	}

	Mp7JrsEnumerator(const Mp7JrsEnumerator<C>& e)
		: content_length(0)
		, current_outpos(0)
		, max_outpos(0)
		, content(0)
	{
		content = allocateVector(e.content_length);
		content_length = e.content_length;
		unsigned int i;
		for (i = 0; i < e.Size(); i++) {
			content[i] = e.content[i];
		}
		current_outpos = e.current_outpos;
		max_outpos = e.max_outpos;
	}

	~Mp7JrsEnumerator()
	{
		Clear();
		deallocateVector(content);
	}

	Mp7JrsEnumerator<C>& operator= (const Mp7JrsEnumerator<C>& e)
	{
		if (&e != this) {
			// Implement assignment in terms of construction
			Mp7JrsEnumerator<C> temp(e);
			Swap(temp);
		}
		return *this;
	}

	bool operator== (const Mp7JrsEnumerator<C>& e) const
	{
		return this == &e;
	}

	bool operator!= (const Mp7JrsEnumerator<C>& e) const
	{
		return !operator==(e);
	}

	/** Checks if there are more elements in the enumerator when iterating in forward direction.
	  * @return True if there are more elements, false otherwise. */
	bool HasNext() const
	{
		return (current_outpos < Size());
	}

	/** Checks if there are more elements in the enumerator when iterating in reverse direction.
	  * @return True if there are more elements, false otherwise. */
	bool HasPrevious() const
	{
		return current_outpos > 0;
	}

	/** Gets the next element.
	  * @return A smart pointer to the next element. */
	C GetNext()
	{
		if (!HasNext())
			return C();
		else
			return content[current_outpos++];
	}

	/** Gets the previous element.
	  * @return A smart pointer to the previous element. */
	C GetPrevious()
	{
		if (!HasPrevious())
			return C();
		else
			return content[current_outpos--];
	}

	/** Gets the element at the specified position.
	  * @param i The position of the element to be retrieved. 
	  * @return The element at the specified position.
	  */
	C ElementAt(unsigned int i) const
	{
		if (i >= 0 && i < Size())
			return content[i];
		else
			return C();
	}

	/** Gets the first element.
	  */
	C GetFirst() const
	{ 
		return ElementAt(0); 
	}

	/** Gets the last element.
	  */
	C GetLast() const
	{ 
		return ElementAt(Size() - 1); 
	}

	/** Removes all elements from the enumerator.
	  */
	void Clear()
	{
		unsigned int i;
		for (i=0; i < Size(); i++) {
			// Overwrite vector so that destructors of stored elements
			// are called
			content[i] = C();
		}

		max_outpos = 0;
		current_outpos = 0;
	}

	void Insert(const C &item) 
	{
		if (max_outpos >= content_length) {
			unsigned int new_content_length;
			C *new_content;
			unsigned int i;

			new_content_length = (unsigned int)((content_length + 1) * 1.3);
			new_content = allocateVector(new_content_length);
			for (i = 0; i < Size(); i++) {
				new_content[i] = content[i];
			}
			deallocateVector(content);
			content = new_content;
			content_length = new_content_length;
		}
		content[max_outpos++] = item;
	}

	/** Gets the number of elements of the enumerator.
	  */
	unsigned int Size() const
	{
		return max_outpos;
	}

private:

	void Swap(Mp7JrsEnumerator<C>& e)
	{
		C *tmp;
		unsigned int tmp_2;

		tmp = content;
		content = e.content;
		e.content = tmp;

		tmp_2 = current_outpos;
		current_outpos = e.current_outpos;
		e.current_outpos = tmp_2;
		tmp_2 = max_outpos;
		max_outpos = e.max_outpos;
		e.max_outpos = tmp_2;
		tmp_2 = content_length;
		content_length = e.content_length;
		e.content_length = tmp_2;
	}

private:

	Mp7JrsPtr<Mp7JrsNode> *content;
	unsigned int content_length, max_outpos, current_outpos;
};

typedef Mp7JrsEnumerator< Mp7JrsPtr<Mp7JrsNode> > Mp7JrsNodeEnum;

#endif // MP7JRS_ENUMERATOR_INCLUDED
